@@ -8,30 +8,40 @@ Q_rows = function(df, fun, const=list(), export=list(), seed=128965,
         fail_on_error=TRUE, workers=NULL,
         log_worker=FALSE, wait_time=NA, chunk_size=NA) {
 
-    # basic variable checking
+    # check if call args make sense
+    if (!is.null(memory))
+        template$memory = memory
+    if (!is.null(template$memory) && template$memory < 50)
+        stop("Worker needs about 23 MB overhead, set memory>=50")
+    if (is.na(seed) || length(seed) != 1)
+        stop("'seed' needs to be a length-1 integer")
+
     fun = match.fun(fun)
     df = as.data.frame(df, check.names=FALSE, stringsAsFactors=FALSE)
     n_calls = nrow(df)
-    n_jobs = Reduce(min, c(ceiling(n_calls / job_size), n_jobs, n_calls))
     seed = as.integer(seed)
     check_args(fun, df, const)
+    data = list(fun=fun, const=const, export=export, common_seed=seed)
 
-    if (!is.null(workers)) {
+    # set up qsys if no workers provided
+    if (is.null(workers)) {
+        qsys_id = toupper(qsys_default)
+        if (qsys_id != "LOCAL" && is.null(n_jobs) && is.null(job_size))
+            stop("n_jobs or job_size is required")
+        n_jobs = Reduce(min, c(ceiling(n_calls / job_size), n_jobs, n_calls))
+
+        qsys = workers(n_jobs, data=data, reuse=FALSE, template=template,
+                       log_worker=log_worker)
+    } else {
         qsys_id = class(workers)[1]
         n_jobs = workers$workers
         job_size = NULL
-    } else
-        qsys_id = toupper(qsys_default)
+        qsys = workers
+        do.call(qsys$set_common_data, data)
+    }
 
-    # check job number and memory
-    if (qsys_id != "LOCAL" && is.null(n_jobs) && is.null(job_size))
-        stop("n_jobs or job_size is required")
-    if (!is.null(memory))
-        template$memory = memory
-    if (!is.null(template$memory) && template$memory < 500)
-        stop("Worker needs about 230 MB overhead, set memory>=500")
-    if (is.na(seed) || length(seed) != 1)
-        stop("'seed' needs to be a length-1 integer")
+    if (!qsys$reusable)
+        on.exit(qsys$cleanup())
 
     # use heuristic for wait and chunk size
     if (is.na(wait_time))
@@ -42,22 +52,12 @@ Q_rows = function(df, fun, const=list(), export=list(), seed=128965,
             1e4 * n_calls / utils::object.size(df)[[1]]
         ))
 
+    # process calls
     if (n_jobs == 0 || qsys_id == "LOCAL") {
         list2env(export, envir=environment(fun))
         re = work_chunk(df=df, fun=fun, const_args=const, common_seed=seed)
         unravel_result(re, fail_on_error=fail_on_error)
     } else {
-        data = list(fun=fun, const=const, export=export, common_seed=seed)
-
-        if (is.null(workers)) {
-            qsys = workers(n_jobs, data=data, reuse=FALSE, template=template,
-                           log_worker=log_worker)
-            on.exit(qsys$cleanup())
-        } else {
-            qsys = workers
-            do.call(qsys$set_common_data, data)
-        }
-
         master(qsys=qsys, iter=df, fail_on_error=fail_on_error,
                wait_time=wait_time, chunk_size=chunk_size)
     }
